pub enum DiskSelection {
Index(usize),
Path(PathBuf),
}Expand description
A DiskSelection enumeration is used to select a disk image by either index or path when dealing with containers that contain multiple disk images.
Variants