pub enum RotationDirection {
Clockwise,
CounterClockwise,
}Expand description
Determines the direction of disk surface rotation for visualization functions. Typically, Side 0, the bottom-facing side of a disk, rotates counter-clockwise when viewed from the bottom, and Side 1, the top-facing side, rotates clockwise.
Variants
Implementations